function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import React, { useState, useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import { genUniqueName } from "../../utilities/file";
import { add, swap } from "../../store/fileSlice";
import { reset, select, target, untarget } from "../../store/dragSlice";
import { PlusIcon } from "../icons";
import { InteractiveFileBlock, StaticFileBlock } from "./FileBlock";
import * as __styles from "./FileList.module.css";
var styles = __styles["default"] ? __styles["default"] : __styles;

function FileList() {
  var _useState = useState(),
      _useState2 = _slicedToArray(_useState, 2),
      ShadowElement = _useState2[0],
      setShadowElement = _useState2[1];

  var _files = useSelector(function (state) {
    return state.files.container;
  });

  var _drag = useSelector(function (state) {
    return state.drag;
  });

  var names = ["App"].concat(Object.keys(_files));
  var dispatch = useDispatch();

  var addFile = function addFile() {
    var counts = names.length + 1;
    var name = genUniqueName(names, "Component".concat(counts));
    dispatch(add(name));
  };

  var generateShadow = function generateShadow(name, x, y) {
    var style = {};
    style.position = "absolute";
    style.top = y + 5 + "px";
    style.left = x + 5 + "px";
    style.zIndex = 100;
    style.opacity = 0.5;
    setShadowElement( /*#__PURE__*/React.createElement(StaticFileBlock, {
      name: name,
      style: style
    }));
  };

  var selectCell = function selectCell(event, name) {
    var clientX = event.clientX,
        clientY = event.clientY;
    generateShadow(name, clientX, clientY);
    dispatch(select(name));
  };

  var targetCell = function targetCell(name) {
    return dispatch(target(name));
  };

  var dragCell = function dragCell(event) {
    var from = _drag.from;

    if (from) {
      var _target = event.target,
          clientX = event.clientX,
          clientY = event.clientY;
      generateShadow(from, clientX, clientY); // outside of the file container

      if (_target.id === styles["screenMask"]) {
        return dispatch(untarget());
      }
    }
  };

  var applyDrag = function applyDrag() {
    var from = _drag.from,
        to = _drag.to;

    if (from) {
      resetListener();
      setShadowElement(undefined);

      if (to) {
        dispatch(swap({
          from: from,
          to: to
        }));
      }

      dispatch(reset());
    }
  };

  var resetListener = function resetListener() {
    document.body.removeEventListener("mousemove", dragCell);
    document.body.removeEventListener("mouseup", applyDrag);
  };

  useEffect(function () {
    var from = _drag.from;

    if (from) {
      document.body.addEventListener("mousemove", dragCell);
      document.body.addEventListener("mouseup", applyDrag);
      return function () {
        resetListener();
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [_drag]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    id: styles["filenameContainer"]
  }, names.map(function (name) {
    return /*#__PURE__*/React.createElement(InteractiveFileBlock, {
      key: name,
      name: name,
      onDrag: selectCell,
      onTargeted: targetCell
    });
  }), /*#__PURE__*/React.createElement("div", {
    className: styles["innerContainer"],
    onClick: addFile
  }, /*#__PURE__*/React.createElement("div", {
    className: styles["addBtn"]
  }, /*#__PURE__*/React.createElement(PlusIcon, null)))), ShadowElement, _drag.from && /*#__PURE__*/React.createElement("div", {
    id: styles["screenMask"]
  }));
}

export default FileList;