"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tabbed = tabbed;
exports.packing = packing;
exports.initTranspile = initTranspile;
exports.transpileReact = transpileReact;
exports.transpileVue = transpileVue;
exports.transpileVueApp = transpileVueApp;
exports.transpileReactApp = transpileReactApp;
exports.importModule = importModule;
exports.exportModule = exportModule;
exports.bundleFile = bundleFile;
exports.bundleModule = bundleModule;

var _standalone = require("@babel/standalone");

var _compiler = require("../contants/compiler");

var _compiler2 = require("../models/compiler");

var _regex = require("./regex");

var _transpiler, _appTranspiler;

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function tabbed(code) {
  return "  " + code.replaceAll("\n", "\n  ");
}

function packing(code, _ref) {
  var context = _ref.context,
      fileName = _ref.fileName;
  return "editorContext[\"".concat(fileName, ".js\"] = () => {\n").concat(code, "\n  return self[\"").concat(context, "\"][\"").concat(fileName, ".js\"]\n}");
}

function initTranspile(option) {
  var _ref2 = option,
      context = _ref2.context,
      fileName = _ref2.fileName; // Local module dependency mapping

  return "self[\"".concat(context, "\"][\"").concat(fileName, "\"] = {}");
}

function transpileReact(code) {
  return (0, _standalone.transform)(code, {
    presets: ["es2015", "react"]
  }).code || "";
}

function transpileVue(code) {
  return (0, _standalone.transform)(code, {
    presets: ["es2015", require("@vue/babel-preset-jsx")],
    plugins: [[require("@vue/babel-plugin-jsx"), {
      optimize: true
    }], require("@vue/babel-sugar-v-on")]
  }).code || "";
}

var transpiler = (_transpiler = {}, _defineProperty(_transpiler, _compiler2.TargetFramework.REACT, transpileReact), _defineProperty(_transpiler, _compiler2.TargetFramework.VUE, transpileVue), _transpiler); // NOTE: Still not stable

function transpileVueApp(appCode) {
  var code = "\n" + transpileVue(appCode);
  code += '\nreturn self["vue"].createApp(App);';
  return code;
}

function transpileReactApp(appCode) {
  var code = "\n" + transpileReact(appCode);
  code += "\nreturn React.createElement(App, null);";
  return code;
}

var appTranspiler = (_appTranspiler = {}, _defineProperty(_appTranspiler, _compiler2.TargetFramework.REACT, transpileReactApp), _defineProperty(_appTranspiler, _compiler2.TargetFramework.VUE, transpileVueApp), _appTranspiler);

function importModule(code, context) {
  var _REQUIRE_MODULE_GETTE = _slicedToArray(_compiler.REQUIRE_MODULE_GETTER_REGEXES, 2),
      ROOT_GETTER = _REQUIRE_MODULE_GETTE[0],
      RELATIVE_GETTER = _REQUIRE_MODULE_GETTE[1];

  return code.replaceAll(_compiler.REQUIRE_KEYWORD_REGEX, function (pattern) {
    try {
      var packageName = (0, _regex.matchOrEmpty)(ROOT_GETTER, pattern)[1];
      return "self[\"".concat(packageName, "\"]");
    } catch (_unused) {
      var fileName = (0, _regex.matchOrEmpty)(RELATIVE_GETTER, pattern)[1];
      return "self[\"".concat(context, "\"][\"").concat(fileName, "\"] ? self[\"").concat(context, "\"][\"").concat(fileName, "\"] : editorContext[\"").concat(fileName, "\"]()");
    }
  });
}

function exportModule(code, _ref3) {
  var context = _ref3.context,
      fileName = _ref3.fileName;
  // Replace exports init
  code = code.replace('Object.defineProperty(exports, "__esModule", {\n  value: true\n});', "self[\"".concat(context, "\"][\"").concat(fileName, ".js\"] = {}")); // Replaces `exports.<something>`

  code = code.replaceAll(_compiler.EXPORT_KEYWORD_REGEX, function (pattern) {
    var moduleName = (0, _regex.matchOrEmpty)(_compiler.EXPORT_MODULE_GETTER_REGEX, pattern)[1];
    return "self[\"".concat(context, "\"][\"").concat(fileName, ".js\"][\"").concat(moduleName, "\"]");
  });
  return code;
}

function bundleFile(code, option) {
  initTranspile(option);
  code = transpiler[option.target](code);
  code = exportModule(code, option);
  code = tabbed(code);
  code = packing(code, option);
  return code;
}

function bundleModule(_ref4, option) {
  var app = _ref4.app,
      files = _ref4.files;
  var appOnly = files === undefined;
  var context = option.context,
      allowDependencies = option.allowDependencies,
      target = option.target;
  var bundle = "delete self[\"".concat(context, "\"]; self[\"").concat(context, "\"] = {}; var editorContext = {};\n");

  if (!appOnly) {
    // Iterate additional files
    Object.keys(files).forEach(function (name) {
      bundle += "\n" + bundleFile(files[name], {
        context: context,
        target: target,
        fileName: name
      });
    });
  } // Main app file


  bundle += appTranspiler[target](app);

  if (allowDependencies) {
    // Handle package imports (replaces `require("<something>")`)
    bundle = importModule(bundle, context);
  }
  /* eslint-disable no-new-func */


  return bundle;
}
//# sourceMappingURL=compiler.js.map
